"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cacheConfig_1 = require("./cacheConfig");
const logger_1 = require("../logger");
const salt_1 = require("./salt");
const backfill = __importStar(require("backfill/lib/api"));
const path_1 = __importDefault(require("path"));
async function cacheHash(task, info, root, config) {
    const packagePath = path_1.default.dirname(info.packageJsonPath);
    const cacheConfig = cacheConfig_1.getCacheConfig(packagePath, config);
    const backfillLogger = backfill.makeLogger("error", process.stdout, process.stderr);
    const name = info.name;
    const hashKey = salt_1.salt(config.cacheOptions.environmentGlob || ["lage.config.js"], `${info.name}|${task}|${JSON.stringify(config.args)}`, root);
    backfillLogger.setName(name);
    try {
        return await backfill.computeHash(packagePath, backfillLogger, hashKey, cacheConfig);
    }
    catch (_a) {
        // computeHash can throw exception when git is not installed or the repo hashes cannot be calculated with a staged file that is deleted
        // lage will continue as if this package cannot be cached
    }
    return null;
}
exports.cacheHash = cacheHash;
async function cacheFetch(hash, info, config) {
    if (!hash) {
        return false;
    }
    const packagePath = path_1.default.dirname(info.packageJsonPath);
    const cacheConfig = cacheConfig_1.getCacheConfig(packagePath, config);
    const backfillLogger = backfill.makeLogger("error", process.stdout, process.stderr);
    try {
        return await backfill.fetch(packagePath, hash, backfillLogger, cacheConfig);
    }
    catch (e) {
        logger_1.logger.error(`${info.name} fetchBackfill`, e);
    }
    return false;
}
exports.cacheFetch = cacheFetch;
async function cachePut(hash, info, config) {
    if (!hash) {
        return;
    }
    const packagePath = path_1.default.dirname(info.packageJsonPath);
    const cacheConfig = cacheConfig_1.getCacheConfig(packagePath, config);
    const backfillLogger = backfill.makeLogger("warn", process.stdout, process.stderr);
    try {
        await backfill.put(packagePath, hash, backfillLogger, cacheConfig);
    }
    catch (e) {
        // sometimes outputGlob don't match any files, so skipping this
    }
}
exports.cachePut = cachePut;
