"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const filterPackages_1 = require("./filterPackages");
const workspace_tools_1 = require("workspace-tools");
const git_1 = require("../git");
const fg = __importStar(require("fast-glob"));
function getPipelinePackages(workspace, config) {
    const { scope, since, repoWideChanges, includeDependencies } = config;
    // If scoped is defined, get scoped packages
    const hasScopes = Array.isArray(scope) && scope.length > 0;
    let scopedPackages = undefined;
    if (hasScopes) {
        scopedPackages = workspace_tools_1.getScopedPackages(scope, workspace.allPackages);
    }
    const hasSince = typeof since !== "undefined";
    let changedPackages = undefined;
    // Be specific with the changed packages only if no repo-wide changes occurred
    if (hasSince && !hasRepoChanged(since, workspace.root, repoWideChanges)) {
        changedPackages = workspace_tools_1.getChangedPackages(workspace.root, since, config.ignore);
    }
    return filterPackages_1.filterPackages({
        allPackages: workspace.allPackages,
        deps: config.deps,
        scopedPackages,
        changedPackages,
        includeDependencies,
    });
}
exports.getPipelinePackages = getPipelinePackages;
function hasRepoChanged(since, root, environmentGlob) {
    const changedFiles = git_1.getChanges(since, root);
    const envFiles = fg.sync(environmentGlob, { cwd: root });
    let repoWideChanged = false;
    if (changedFiles) {
        for (const change of changedFiles) {
            if (envFiles.includes(change)) {
                repoWideChanged = true;
                break;
            }
        }
    }
    return repoWideChanged;
}
